; following Oregonian's tutorial on Double Bump Map
; found at link: http://gimpchat.com/viewtopic.php?f=23&t=12
; author: Tin Tran
; date: 2014
(define (script-fu-double-bump-map image layer 
              color
			  pattern
              grow-distance
			  blur-radius
			  bump-azimuth
			  bump-elevation
			  bump-depth
			  ;shadow settings
			  shadow-offset-X   ;offset x
			  shadow-offset-Y   ;offset y
			  shadow-blur-radius ;blur-radius
			  shadow-color       ;color
	          shadow-opacity     ;opacity
         )
	(let* 
		(
		(dummy-variable-to-make-let-happy 0)
		(image-width)
		(image-height)

		;vectors/paths related variables to create our own path and select it to cut after blur
		(get-vectors-returned-values)
		(vectors-count)
		(vectors-array)
		(current-vector)

		(color-layer)
		(pattern-layer)
		) ;end of variable declaration
		(gimp-context-push)
		(gimp-image-undo-group-start image)                   ;undo-group in one step


		(set! image-width (car (gimp-image-width image)))
		(set! image-height (car (gimp-image-height image)))
		;START
		(gimp-layer-resize-to-image-size layer)
		(gimp-image-select-item image CHANNEL-OP-REPLACE layer)
		(gimp-edit-fill layer FILL-WHITE)

		;grow selection, invert, the create vector path
		(gimp-selection-grow image grow-distance)
		(gimp-selection-invert image)
		(plug-in-sel2path RUN-NONINTERACTIVE image layer)
		(set! get-vectors-returned-values (gimp-image-get-vectors image))
		(set! vectors-count (car  get-vectors-returned-values))
		(set! vectors-array (cadr get-vectors-returned-values))
		(set! current-vector (aref vectors-array 0))      ;top path, our own created from selection

		;blurs white bump map
		(gimp-selection-none image)
		(plug-in-gauss RUN-NONINTERACTIVE image layer
											blur-radius blur-radius
											1 ;1 RLE or 0 IIR
											)
											
		;creates new colored layer to bump map
		(set! color-layer (car (gimp-layer-new image image-width image-height
							   RGBA-IMAGE "Color" 100 13)))  ;creates layer COLOR-MODE (13)
		;creates new pattern layer to bump map as well
		(set! pattern-layer (car (gimp-layer-new image image-width image-height
							   RGBA-IMAGE "Pattern" 100 0)))  ;creates layer LAYER-MODE-NORMAL-LEGACY
		;insert above current layer  
		(gimp-image-insert-layer image color-layer 0 (car (gimp-image-get-item-position image layer)))
		(gimp-image-insert-layer image pattern-layer 0 (car (gimp-image-get-item-position image layer)))
		
		;fill color layer with color
		(gimp-image-set-active-layer image color-layer)
		(gimp-selection-all image)
		(gimp-context-set-foreground color)
		(gimp-edit-fill color-layer FILL-FOREGROUND)
		
		;fill pattern layer with pattern
		(gimp-image-set-active-layer image pattern-layer)
		(gimp-selection-all image)
		(gimp-context-set-pattern pattern)
		(gimp-edit-fill pattern-layer FILL-PATTERN)
			
		
		;perform bump map 1st time (color layer)
		(plug-in-bump-map 1 image color-layer layer 
							  bump-azimuth ;Azimuth
							  bump-elevation  ;Elevation
							  bump-depth  ;Depth
							  0   ;X-offset
							  0   ;Y-offset
							  0   ;water level
							  0   ;ambient
							  TRUE ;compensate for darkening
							  FALSE ;invert
							  0    ;type of map LINEAR (0), SPHERICAL (1), SINUSOIDAL (2)
		)
		;perform bump map 1st time (pattern layer)
		(plug-in-bump-map 1 image pattern-layer layer 
							  bump-azimuth ;Azimuth
							  bump-elevation  ;Elevation
							  bump-depth  ;Depth
							  0   ;X-offset
							  0   ;Y-offset
							  0   ;water level
							  0   ;ambient
							  TRUE ;compensate for darkening
							  FALSE ;invert
							  0    ;type of map LINEAR (0), SPHERICAL (1), SINUSOIDAL (2)
		)

		;cut out color layer using previously created path
		(gimp-image-select-item image CHANNEL-OP-REPLACE current-vector)
		(gimp-edit-cut color-layer)
		(gimp-selection-none image)
		;cur out pattern layer using previously created path
		(gimp-image-select-item image CHANNEL-OP-REPLACE current-vector)
		(gimp-edit-cut pattern-layer)
		(gimp-selection-none image)
		
		;delete current-vector
		(gimp-image-remove-vectors image current-vector)
				
		;bump map 2nd time (color layer)
		(plug-in-bump-map 1 image color-layer layer 
							  bump-azimuth ;Azimuth
							  bump-elevation  ;Elevation
							  bump-depth  ;Depth
							  0   ;X-offset
							  0   ;Y-offset
							  0   ;water level
							  0   ;ambient
							  TRUE ;compensate for darkening
							  FALSE ;invert
							  0    ;type of map LINEAR (0), SPHERICAL (1), SINUSOIDAL (2)
		)
        ;bump map 2nd time (pattern layer)
		(plug-in-bump-map 1 image pattern-layer layer 
							  bump-azimuth ;Azimuth
							  bump-elevation  ;Elevation
							  bump-depth  ;Depth
							  0   ;X-offset
							  0   ;Y-offset
							  0   ;water level
							  0   ;ambient
							  TRUE ;compensate for darkening
							  FALSE ;invert
							  0    ;type of map LINEAR (0), SPHERICAL (1), SINUSOIDAL (2)
		)
		;drops shadow of color-layer
		(script-fu-drop-shadow image pattern-layer
								   shadow-offset-X   ;offset x
								   shadow-offset-Y   ;offset y
								   shadow-blur-radius ;blur-radius
								   shadow-color       ;color
								   shadow-opacity     ;opacity
								   TRUE ;allow resizing
		)
	  ;DONE
	  
	
	(gimp-image-undo-group-end image)                     ;undo group in one step
	(gimp-context-pop)
	(gimp-displays-flush)
	) ;end of let*
) ;end of define

(script-fu-register
  "script-fu-double-bump-map"         ;function name
  "<Image>/Filters/Alpha to Logo/Double Bump Map..."    ;menu register
  "Creates Double Bump Map effect on your art/text/shapes."       ;description
  "Tin Tran"                          ;author name
  "copyright info and description"         ;copyright info or description
  "2014"                          ;date
  "RGB*, GRAY*"                        ;mode
  SF-IMAGE      "Image" 0                   
  SF-DRAWABLE   "Layer" 0
  SF-COLOR      "Solid Color"          '(0 129 229)
  SF-PATTERN    "Pattern" (car (gimp-context-get-pattern))
  ;SF-TOGGLE     "Use Pattern instead of solid color" TRUE
  SF-ADJUSTMENT "Grow Distance(from border to cut)" '(2 0 100 1 10 0 0)
  SF-ADJUSTMENT "Bump Map Blur Radius"             '(3 0 100 1 10 0 0)
  SF-ADJUSTMENT "Bump Azimuth"             '(135 0 360 1 10 0 0)
  SF-ADJUSTMENT "Bump Elevation"           '(33 0 100 1 10 0 0)
  SF-ADJUSTMENT "Bump Depth"               '(35 0 100 1 10 0 0)
  
  SF-ADJUSTMENT "Shadow Offset X" '(4 0 100 1 10 0 0)
  SF-ADJUSTMENT "Shadow Offset Y" '(4 0 100 1 10 0 0)
  SF-ADJUSTMENT "Shadow Blur Radius" '(15 0 100 1 10 0 0)
  SF-COLOR      "Shadow Color"     '(0 0 0)
  SF-ADJUSTMENT "Shadow Opacity"   '(60 0 100 1 10 0 0)
  
)